/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.ws.resource.lifetime.impl;

import java.text.ParseException;
import java.util.Date;

import javax.xml.namespace.QName;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.ws.resource.lifetime.WsrlConstants;
import org.apache.muse.util.xml.XmlSerializable;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.util.xml.XsdUtils;

/**
 *
 * SetTerminationTimeResponse is a serializer/deserializer for the 
 * WS-ImmediateTermination SetTerminationTime operation's response content.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class SetTerminationTimeResponse implements XmlSerializable
{
    //
    // The wsrl:CurrentTime
    //
    private Date _currentTime = null;
    
    //
    // Used to lookup all exception messages
    //
    private Messages _MESSAGES = 
        MessagesFactory.get(SetTerminationTimeResponse.class);
    
    //
    // The resource's new wsrl:TerminationTime (could be null)
    //
    private Date _terminationTime = null;
    
    public SetTerminationTimeResponse(Date termination)
    {
        _currentTime = new Date();
        _terminationTime = termination;
    }
    
    public SetTerminationTimeResponse(Element xml)
        throws ParseException
    {
        if (xml == null)
            throw new NullPointerException(_MESSAGES.get("NullResponseElement"));
        
        //
        // current time on the RESOURCE (DON'T use 'new Date()')
        //
        QName qname = WsrlConstants.CURRENT_TIME_QNAME;
        String timeString = XmlUtils.getElementText(xml, qname);
        _currentTime = XsdUtils.getLocalTime(timeString);
        
        //
        // termination time that was just set
        //
        qname = WsrlConstants.NEW_TIME_QNAME;
        timeString = XmlUtils.getElementText(xml, qname);
        
        //
        // if the termination was cancelled, there won't be a value
        //
        if (timeString != null)
            _terminationTime = XsdUtils.getLocalTime(timeString);
    }
    
    public Date getCurrentTime()
    {
        return _currentTime;
    }
    
    public Date getTerminationTime()
    {
        return _terminationTime;
    }
    
    public String toString()
    {
        return XmlUtils.toString(toXML(), false);
    }
    
    public Element toXML()
    {
        return toXML(XmlUtils.createDocument());
    }
    
    public Element toXML(Document doc)
    {
        if (doc == null)
            throw new NullPointerException(_MESSAGES.get("NullDocument"));
        
        Element root = XmlUtils.createElement(doc, WsrlConstants.SET_TERMINATION_RESPONSE_QNAME);
        XmlUtils.setElement(root, WsrlConstants.NEW_TIME_QNAME, getTerminationTime());
        XmlUtils.setElement(root, WsrlConstants.CURRENT_TIME_QNAME, getCurrentTime());
        
        return root;
    }
}
