/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.ws.resource.metadata.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.xml.namespace.QName;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlSerializable;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.resource.metadata.WsrmdConstants;

/**
 *
 * PropertyMetadata is an internal class that represents one metadata 
 * definition (wsrmd:Property) in a WS-RMD document.
 * 
 * @author Dan Jemiolo (danj)
 *
 * @see SimpleMetadataDescriptor
 * 
 */

class PropertyMetadata implements XmlSerializable
{
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = MessagesFactory.get(PropertyMetadata.class);

    //
    // Modifiability values
    //    
    public static final int READ_ONLY = 1;
    public static final int READ_WRITE = 2; 
    
    //
    // Mutability values
    //    
    public static final int APPENDABLE = 3;
    public static final int CONSTANT = 4; 
    public static final int MUTABLE = 5;
    
    //
    // extra (non-standard) metadata values defined by the application
    //
    private Map _extendedMetadata = null;
    
    //
    // The initial values for this property - all instances of the resource 
    // should have these values for the property after initialization. If 
    // the property is mutable, the values may be deleted after that time.
    //
    private Collection _initialValues = null;
    
    //
    // The modifiability rule for the property. The default is read-write.
    //
    private int _modifiability = READ_WRITE;
    
    //
    // The mutability rule for this property. The default is mutable.
    //
    private int _mutability = MUTABLE;
    
    //
    // The name of the property being defined
    //
    private QName _qname = null;
    
    //
    // The static values for this property - all instances of the resource 
    // should have these values for the property after initialization. The 
    // values CANNOT be deleted, even if the property is read-write/mutable.
    //
    private Collection _staticValues = null;
    
    //
    // The valid values for this property - if this is empty, all values 
    // are allowed
    //
    private Collection _validValues = null;
    
    //
    // The lower-bound of the valid value range (or null, if there is no 
    // value range). This is in string form, and must be converted to the 
    // appropriate data type by the caller.
    //
    private String _valueLowerBound = null;
    
    //
    // The upper-bound of the valid value range (or null, if there is no 
    // value range). This is in string form, and must be converted to the 
    // appropriate data type by the caller.
    //
    private String _valueUpperBound = null;
    
    public PropertyMetadata(QName qname, String modifiability, String mutability)
    {
        if (qname == null)
            throw new NullPointerException(_MESSAGES.get("NullPropertyName"));

        _qname = qname;
        setModifiability(modifiability);
        setMutability(mutability);
        
        _extendedMetadata = new HashMap();
        setInitialValues(Collections.EMPTY_LIST);
        setStaticValues(Collections.EMPTY_LIST);
        setValidValues(Collections.EMPTY_LIST);
    }
    
    /**
     * 
     * Creates a new metadata definition from the given XML.
     *
     * @param property
     *        The property's metadata definition, in XML form.
     *
     */
    public PropertyMetadata(Element property)
    {
        if (property == null)
            throw new NullPointerException(_MESSAGES.get("NullPropertyElement"));
        
        //
        // the path QName maps to a WS-RP property definition
        //
        String nameString = property.getAttribute(WsrmdConstants.NAME);
        
        if (nameString == null || nameString.length() == 0)
            throw new RuntimeException(_MESSAGES.get("NoPropertyName"));
        
        _qname = XmlUtils.parseQName(nameString, property);
        
        _modifiability = parseModifiability(property);
        _mutability = parseMutability(property);
        
        //
        // make sure constant properties are also read-only!
        //
        if ((_mutability == CONSTANT) && (_modifiability != READ_ONLY))
        {
            Object[] filler = { _qname };
            String message = _MESSAGES.get("ConstantButNotReadOnly", filler);
            throw new IllegalStateException(message);
        }
        
        //
        // initial value set (optional)
        //
        _initialValues = parseInitialValues(property);
        
        //
        // static value set (optional)
        //
        _staticValues = parseStaticValues(property);
        
        //
        // valid value set (optional)
        //
        _validValues = parseValidValues(property);
        
        //
        // valid value range (optional)
        //
        Element rangeXML = XmlUtils.getElement(property, WsrmdConstants.VALID_RANGE_QNAME);
        
        if (rangeXML != null)
        {
            _valueLowerBound = rangeXML.getAttribute(WsrmdConstants.LOWER_BOUND);
            _valueUpperBound = rangeXML.getAttribute(WsrmdConstants.UPPER_BOUND);
            
            //
            // for consistency, if there was no bound, we want the value 
            // to be null
            //
            if (_valueLowerBound != null && _valueLowerBound.length() == 0)
                _valueLowerBound = null;

            if (_valueUpperBound != null && _valueUpperBound.length() == 0)
                _valueUpperBound = null;            
        }
        
        _extendedMetadata = parseExtendedMetadata(property);
    }
    
    public String getExtendedMetadata(QName elementName)
    {
        return (String)_extendedMetadata.get(elementName);
    }
    
    public Collection getExtendedMetadataNames()
    {
        return Collections.unmodifiableSet(_extendedMetadata.keySet());
    }
    
    public Collection getInitialValues()
    {
        return Collections.unmodifiableCollection(_initialValues);
    }
    
    public String getLowerBound()
    {
        return _valueLowerBound;
    }
    
    public int getModifiability()
    {
        return _modifiability;
    }
    
    public String getModifiabilityString()
    {
        return _modifiability == READ_ONLY ? WsrmdConstants.READ_ONLY : WsrmdConstants.READ_WRITE;
    }
    
    public int getMutability()
    {
        return _mutability;
    }
    
    public String getMutabilityString()
    {
        if (_modifiability == CONSTANT)
            return WsrmdConstants.CONSTANT;
        
        else if (_modifiability == APPENDABLE)
            return WsrmdConstants.APPENDABLE;
        
        return WsrmdConstants.MUTABLE;
    }

    public QName getPropertyName()
    {
        return _qname;
    }
    
    public Collection getStaticValues()
    {
        return Collections.unmodifiableCollection(_staticValues);
    }
    
    public String getUpperBound()
    {
        return _valueUpperBound;
    }
    
    public Collection getValidValues()
    {
        return Collections.unmodifiableCollection(_validValues);
    }
    
    private boolean isElementInCollection(Collection setOfValues, Object value)
    {        
        //
        // complex type values will be XML fragments. in order to have 
        // a simple equality check, we create an XML instance of the 
        // value being tested and compare it to the XML fragments in the 
        // value set:
        //
        
        //
        // 1. create XML instance of the value being tested. for complex 
        //    types, the value is already XML; for simple types we build:
        //
        //    <PropertyName>value</PropertyName>
        //
        Element valueXML = null;
        
        if (value instanceof Element)
            valueXML = (Element)value;
        
        else
        {
            QName qname = getPropertyName();
            valueXML = XmlUtils.createElement(XmlUtils.createDocument(), qname, value);
        }
        
        //
        // 2. compare the XML fragments
        //
        Iterator i = setOfValues.iterator();
        
        while (i.hasNext())
        {
            Element next = (Element)i.next();
            
            if (XmlUtils.equals(next, valueXML))
                return true;
        }
        
        //
        // 3. no matches - invalid
        //
        return false;
    }
    
    public boolean isInitialValue(Object value)
    {
        return isElementInCollection(_initialValues, value);
    }
    
    public boolean isStaticValue(Object value)
    {
        return isElementInCollection(_staticValues, value);
    }
    
    public boolean isValidValue(Object value)
    {
        //
        // if no valid values were specified, it means ANY value is ok
        //
        if (_validValues.isEmpty())
            return true;
        
        return isElementInCollection(_validValues, value);
    }
    
    private Collection parseElementCollection(Element property, QName qname)
    {
        Collection values = new ArrayList();
        
        Element valuesXML = XmlUtils.getElement(property, qname);
        
        if (valuesXML != null)
        {
            //
            // we store the values as XML fragments - we will need 
            // these to do simple validation later. converting the 
            // values to non-XML would serve no purpose, and would 
            // require much more complexity
            //
            Element[] children = XmlUtils.getAllElements(valuesXML);
            
            for (int n = 0; n < children.length; ++n)
                values.add(children[n]);
        }
        
        return values;
    }
    
    private Map parseExtendedMetadata(Element property)
    {
        Map extended = new HashMap();
        
        Element[] children = XmlUtils.getAllElements(property);
        
        for (int n = 0; n < children.length; ++n)
        {
            QName qname = XmlUtils.getElementQName(children[n]);
            
            //
            // 'extended metadata' is any element that isn't the 
            // RMD *Values elements
            //
            if (!qname.equals(WsrmdConstants.VALID_VALUES_QNAME) && 
                !qname.equals(WsrmdConstants.VALID_RANGE_QNAME) && 
                !qname.equals(WsrmdConstants.STATIC_VALUES_QNAME)&& 
                !qname.equals(WsrmdConstants.INITIAL_VALUES_QNAME))
            {
                String value = XmlUtils.extractText(children[n]);
                extended.put(qname, value);
            }
        }
        
        return extended;
    }
    
    private Collection parseInitialValues(Element property)
    {
        return parseElementCollection(property, WsrmdConstants.INITIAL_VALUES_QNAME);
    }
    
    private int parseModifiability(Element property)
    {
        String value = property.getAttribute(WsrmdConstants.MODIFIABILITY);
        
        //
        // default value is 'read-write'
        //
        if (value == null || value.length() == 0)
            return READ_WRITE;
        
        if (value.equals(WsrmdConstants.READ_ONLY))
            return READ_ONLY;
        
        else if (value.equals(WsrmdConstants.READ_WRITE))
            return READ_WRITE;
        
        String name = property.getAttribute(WsrmdConstants.NAME);
        Object[] filler = { name, value };
        throw new RuntimeException(_MESSAGES.get("InvalidModifiability", filler));
    }

    private int parseMutability(Element property)
    {
        String value = property.getAttribute(WsrmdConstants.MUTABILITY);
        
        //
        // default value is 'mutable'
        //
        if (value == null || value.length() == 0)
            return MUTABLE;
        
        if (value.equals(WsrmdConstants.APPENDABLE))
            return APPENDABLE;
        
        else if (value.equals(WsrmdConstants.CONSTANT))
            return CONSTANT;
        
        else if (value.equals(WsrmdConstants.MUTABLE))
            return MUTABLE;

        String name = property.getAttribute(WsrmdConstants.NAME);
        Object[] filler = { name, value };
        throw new RuntimeException(_MESSAGES.get("InvalidMutability", filler));
    }
    
    private Collection parseStaticValues(Element property)
    {
        return parseElementCollection(property, WsrmdConstants.STATIC_VALUES_QNAME);
    }
    
    private Collection parseValidValues(Element property)
    {
        return parseElementCollection(property, WsrmdConstants.VALID_VALUES_QNAME);
    }
    
    public void setExtendedMetadata(QName elementName, String value)
    {
        _extendedMetadata.put(elementName, value);
    }
    
    public void setInitialValues(Collection values)
    {
        _initialValues = new ArrayList(values);
    }
    
    public void setLowerBound(String value)
    {
        if (!getValidValues().isEmpty())
        {
            Object[] filler = { getPropertyName() };
            throw new RuntimeException(_MESSAGES.get("ValidValuesExists", filler));
        }
        
        _valueLowerBound = value;
    }
    
    public void setModifiability(String value)
    {
        if (value.equalsIgnoreCase(WsrmdConstants.READ_ONLY))
            _modifiability = READ_ONLY;
        
        else if (value.equalsIgnoreCase(WsrmdConstants.READ_WRITE))
            _modifiability = READ_WRITE;
        
        else
        {
            Object[] filler = { _qname, value };
            throw new RuntimeException(_MESSAGES.get("InvalidModifiability", filler));
        }
    }
    
    public void setMutability(String value)
    {
        if (value.equalsIgnoreCase(WsrmdConstants.CONSTANT))
            _mutability = CONSTANT;
        
        else if (value.equalsIgnoreCase(WsrmdConstants.APPENDABLE))
            _mutability = APPENDABLE;
        
        else if (value.equalsIgnoreCase(WsrmdConstants.MUTABLE))
            _mutability = MUTABLE;
        
        else
        {
            Object[] filler = { _qname, value };
            throw new RuntimeException(_MESSAGES.get("InvalidMutability", filler));
        }
    }
    
    public void setPropertyName(QName propertyName)
    {
        if (propertyName == null)
            throw new NullPointerException(_MESSAGES.get("NullPropertyName"));
        
        _qname = propertyName;
    }
    
    public void setStaticValues(Collection values)
    {
        _staticValues = new ArrayList(values);
    }
    
    public void setUpperBound(String value)
    {
        if (!getValidValues().isEmpty())
        {
            Object[] filler = { getPropertyName() };
            throw new RuntimeException(_MESSAGES.get("ValidValuesExists", filler));
        }
        
        _valueUpperBound = value;
    }
    
    public void setValidValues(Collection values)
    {
        if (getLowerBound() != null || getUpperBound() != null)
        {
            Object[] filler = { getPropertyName(), getLowerBound(), getUpperBound() };
            throw new RuntimeException(_MESSAGES.get("ValidValueRangeExists", filler));
        }
        
        _validValues = new ArrayList(values);
    }

    public Element toXML()
    {
        return toXML(XmlUtils.createDocument());
    }

    public Element toXML(Document doc)
    {
        Element root = XmlUtils.createElement(doc, WsrmdConstants.PROPERTY_QNAME);
        
        QName qname = getPropertyName();
        root.setAttribute(WsrmdConstants.NAME, XmlUtils.toString(qname));
        XmlUtils.setNamespaceAttribute(root, qname.getPrefix(), qname.getNamespaceURI());
        
        root.setAttribute(WsrmdConstants.MODIFIABILITY, getModifiabilityString());        
        root.setAttribute(WsrmdConstants.MUTABILITY, getMutabilityString());
        
        //
        // the ValidValues, StaticValues, and InitialValues are all composed 
        // in a very similar fashion, so we try to loop through them that way
        //

        Element validValues = XmlUtils.createElement(doc, WsrmdConstants.VALID_VALUES_QNAME);
        Element staticValues = XmlUtils.createElement(doc, WsrmdConstants.STATIC_VALUES_QNAME);
        Element initialValues = XmlUtils.createElement(doc, WsrmdConstants.INITIAL_VALUES_QNAME);
        
        Element[] values = new Element[]{ validValues, staticValues, initialValues };
        Collection[] instances = new Collection[] { getValidValues(), getStaticValues(), getInitialValues() };
        
        //
        // almost-hack: we can only have ValidValues or ValidValueRange - not 
        // both. so, assuming we have no ValidValues, we check to see if a 
        // range is provided - it it is, we replace the ValidValues element 
        // with a ValidValueRange element that has the upper/lower bounds.
        //
        String lowerBound = getLowerBound();
        String upperBound = getUpperBound();
        
        if (instances[0].isEmpty() && (lowerBound != null || upperBound != null))
        {
            values[0] = XmlUtils.createElement(doc, WsrmdConstants.VALID_RANGE_QNAME);
            
            if (lowerBound != null)
                values[0].setAttribute(WsrmdConstants.LOWER_BOUND, lowerBound);
            
            if (upperBound != null)
                values[0].setAttribute(WsrmdConstants.UPPER_BOUND, upperBound);
        }
        
        for (int n = 0; n < values.length; ++n)
        {
            root.appendChild(values[n]);
            
            Iterator i = instances[n].iterator();
            
            while (i.hasNext())
            {
                Element next = (Element)i.next();
                next = (Element)doc.importNode(next, true);
                values[n].appendChild(next);
            }
        }
        
        //
        // add custom metadata at the end of the wsrmd:Property
        //
        Iterator i = getExtendedMetadataNames().iterator();
        
        while (i.hasNext())
        {
            QName name = (QName)i.next();
            String value = getExtendedMetadata(name);
            XmlUtils.setElement(root, name, value);
        }
        
        return root;
    }
}
