/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.ws.resource.metadata.impl;

import javax.xml.namespace.QName;

import org.w3c.dom.Element;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.resource.basefaults.BaseFault;
import org.apache.muse.ws.resource.metadata.MetadataDescriptor;
import org.apache.muse.ws.resource.properties.listeners.AbstractChangeApprover;
import org.apache.muse.ws.resource.properties.listeners.PropertyChangeApprover;
import org.apache.muse.ws.resource.properties.set.faults.UnableToModifyResourcePropertyFault;

/**
 *
 * StaticValuesApprover is a 
 * {@linkplain PropertyChangeApprover PropertyChangeApprover} that throws an 
 * exception if a caller tries to delete a property instance whose value is 
 * defined in the WS-RMD document's StaticValues section. If a property's 
 * metadata defines static values, those values must be in the WS-RP document 
 * for the entire resource lifecycle.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class StaticValuesApprover extends AbstractChangeApprover
{
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = 
        MessagesFactory.get(StaticValuesApprover.class);
    
    //
    // The RMD with the static values of the properties
    //
    private MetadataDescriptor _metadata = null;
    
    /**
     * 
     * Creates a new approver for the property with the given name and RMD.
     *
     * @param qname
     *        The QName of the property to monitor.
     * 
     * @param metadata
     *        The RMD descriptor with the metadata for the property.
     *
     */
    public StaticValuesApprover(QName qname, MetadataDescriptor metadata)
    {
        super(qname);

        if (metadata == null)
            throw new NullPointerException(_MESSAGES.get("NullMetadataDescriptor"));
        
        _metadata = metadata;
    }
    
    /**
     * 
     * Throws an exception if the second parameter is null (signifying a 
     * deletion) and the current property value is one of the static values 
     * defined in the RMD.
     *
     */
    public void validateChange(Element oldValue, Element newValue, Object token)
        throws BaseFault
    {
        QName qname = getPropertyName();
        
        //
        // IF we're deleting AND the value is a StaticValue, disallow it
        //
        if ((oldValue != null && newValue == null) && 
            _metadata.isStaticValue(qname, oldValue))
        {
            Object[] filler = { qname, XmlUtils.toString(oldValue, false) };
            throw new UnableToModifyResourcePropertyFault(_MESSAGES.get("DeletingStaticValue", filler));
        }
    }
}
