/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.ws.resource.metadata.impl;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.resource.metadata.MetadataDescriptor;
import org.apache.muse.ws.resource.metadata.WsrmdConstants;

/**
 * 
 * WsrmdUtils is a set of convenience methods related to WS-RMD 1.0.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class WsrmdUtils
{
    /**
     * 
     * This method serializes the given descriptor into XML and then wraps 
     * it in a wsrmd:Definitions element so it can be written or published 
     * as a valid WSRMD document.
     * 
     * @param rmd
     * 
     * @return The XML for the given MetadataDescriptor, wrapped in a 
     *         wsrmd:Definitions element.
     *         
     */
    public static Element createMetadataDocument(MetadataDescriptor rmd)
    {
        Element root = XmlUtils.createElement(WsrmdConstants.DEFINITIONS_QNAME);
        Element rmdXML = rmd.toXML();
        root.appendChild(rmdXML);
        return root;
    }
    
    /**
     * 
     * @param rmd
     *        An XML document containing a root element named wsrmd:Definitions.
     *        
     * @param name
     *        The value of the <em>name</em> attribute for the desired 
     *        metadata descriptor.
     *        
     * @return The metadata descriptor in the document that has the given name, 
     *         or null if the name is not found.
     *
     */
    public static Element getMetadataDescriptor(Document rmd, String name)
    {
        Element root = XmlUtils.getFirstElement(rmd);  // wsrmd:Definitions
        Element[] descriptors = XmlUtils.getElements(root, WsrmdConstants.DESCRIPTOR_QNAME);
        
        for (int n = 0; n < descriptors.length; ++n)
        {
            String nextAttr = descriptors[n].getAttribute(WsrmdConstants.NAME);
            
            if (nextAttr != null && nextAttr.equals(name))
                return descriptors[n];
        }
        
        return null;
    }
}
