/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.ws.resource.properties.get.impl;

import javax.xml.namespace.QName;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlSerializable;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.resource.ext.faults.InvalidMessageFormatFault;
import org.apache.muse.ws.resource.properties.WsrpConstants;

/**
 *
 * GetMultipleRequest is a serializer/deserializer for the WS-ResourceProperties  
 * GetMultipleResourceProperties operation's request content.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class GetMultipleRequest implements XmlSerializable
{
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = MessagesFactory.get(GetMultipleRequest.class);
        
    //
    // The names of the properties to retrieve
    //
    private QName[] _qnames = null;
    
    public GetMultipleRequest(Element request)
        throws InvalidMessageFormatFault
    {
        if (request == null)
            throw new NullPointerException(_MESSAGES.get("NullRequestElement"));
        
        //
        // load the property QNames from the ResourceProperty elements...
        //        
        QName qname = WsrpConstants.PROPERTY_QNAME;
        Element[] properties = XmlUtils.getElements(request, qname);
        
        if (properties.length == 0)
            throw new InvalidMessageFormatFault(_MESSAGES.get("NoPropertiesInGet"));
        
        _qnames = new QName[properties.length];
        
        for (int n = 0; n < properties.length; ++n)
        {
            _qnames[n] = XmlUtils.getQName(properties[n]);
            
            if (_qnames[n] == null)
            {
                Object[] filler = { new Integer(n) };
                throw new InvalidMessageFormatFault(_MESSAGES.get("NullPropertyN", filler));
            }
        }
    }
    
    public GetMultipleRequest(QName[] qnames)
    {
        if (qnames == null)
            throw new NullPointerException(_MESSAGES.get("NullQNameArray"));
        
        for (int n = 0; n < qnames.length; ++n)
        {
            if (qnames[n] == null)
            {
                Object[] filler = { new Integer(n) };
                throw new NullPointerException(_MESSAGES.get("NullQNameN", filler));
            }
        }
            
        _qnames = qnames;
    }
    
    public QName[] getQNames()
    {
        return _qnames;
    }
    
    public String toString()
    {
        return XmlUtils.toString(toXML(), false);
    }
    
    public Element toXML()
    {
        return toXML(XmlUtils.createDocument());
    }
    
    public Element toXML(Document doc)
    {
        if (doc == null)
            throw new NullPointerException(_MESSAGES.get("NullDocument"));
        
        QName qname = WsrpConstants.GET_MULTIPLE_QNAME;
        Element root = XmlUtils.createElement(doc, qname);
        
        qname = WsrpConstants.PROPERTY_QNAME;
        QName[] qnames = getQNames();
        
        for (int n = 0; n < qnames.length; ++n)
        {
            Element property = XmlUtils.createElement(doc, qname, qnames[n]);
            root.appendChild(property);
        }
        
        return root;
    }
}
