/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.ws.resource.properties.query.impl;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.xml.transform.TransformerException;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XPathUtils;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.resource.properties.query.QueryExpression;
import org.apache.muse.ws.resource.properties.query.faults.QueryEvaluationErrorFault;

/**
 * 
 * XPathQueryExpression is an implementation of query evaluations using XPath 1.0.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class XPathQueryExpression implements QueryExpression
{
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = 
        MessagesFactory.get(XPathQueryExpression.class);
    
    public Node[] evaluate(Element wsrpDoc, String query)
        throws QueryEvaluationErrorFault
    {
        //
        // we need to store all of the prefix/URI mappings in the instances 
        // and add them to the root node
        //
        // NOTE: This search is inefficient, as we'll do a lot of redundant 
        //       searching. It is necessary only because WS-RP allows Insert 
        //       to add multiple instances of a property with different 
        //       values. One instance of a property might have an element 
        //       (and a prefix/URI) that is not found in another instance; 
        //       thus, we have to search all children.
        //
        Map namespacesByPrefix = new HashMap();
        
        Element[] valuesXML = XmlUtils.getAllElements(wsrpDoc);
        
        for (int n = 0; n < valuesXML.length; ++n)
        {
            //
            // collect all of the prefix/URI mappings in the subtree
            //
            Map namespaces = XmlUtils.getAllNamespaces(valuesXML[n]);
            namespacesByPrefix.putAll(namespaces);
        }
        
        //
        // add the property's various namespaces to the root element 
        // so we can easily query them with XPath 1.0
        //
        // NOTE: We could write our own PrefixResolver, but it would 
        //       be slower, and doesn't prevent the conflict of having 
        //       two URIs (for two sibling nodes) with the same prefix, 
        //       which is legal XML but will cause XPath to be mixfused.
        //
        //       The small cost of adding one copy of the NS to the root 
        //       makes for a much more efficient/correct system.
        //
        
        Iterator i = namespacesByPrefix.keySet().iterator();
        
        while (i.hasNext())
        {
            String prefix = (String)i.next();
            String namespace = (String)namespacesByPrefix.get(prefix);
            XmlUtils.setNamespaceAttribute(wsrpDoc, prefix, namespace);
        }
        
        try
        {
            return XPathUtils.select(wsrpDoc, query);
        }
        
        catch (TransformerException error)
        {
            Object[] filler = { query, error.getMessage() };
            throw new QueryEvaluationErrorFault(_MESSAGES.get("QueryEvaluationError", filler));
        }
    }
    
    public String getDialect()
    {
        return XPathUtils.NAMESPACE_URI;
    }
}

