/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.ws.resource.properties.schema.impl;

import javax.xml.namespace.QName;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.util.xml.XsdUtils;
import org.apache.muse.ws.resource.WsResourceCapability;
import org.apache.muse.ws.resource.properties.schema.ResourcePropertiesSchema;
import org.apache.muse.ws.wsdl.WsdlUtils;

/**
 *
 * PropertySchemaDefinition is an internal class that represents one type 
 * definition in a WS-RP document definition. It holds all of the schema 
 * information for one resource property.
 *
 * @author Dan Jemiolo (danj)
 *
 * @see SimpleResourcePropertiesSchema
 * 
 */

class PropertySchemaDefinition
{
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = MessagesFactory.get(PropertySchemaDefinition.class);
    
    //
    // the capability that implements the resource property
    //
    private WsResourceCapability _capability = null;
    
    //
    // True if instances of the property can be set to null. The 
    // default is 'false'.
    //
    private boolean _isNillable = false;
    
    //
    // The maximum number of instances per properties document. The 
    // default is 1.
    //
    private int _max = 1;

    //
    // The minimum number of instances per properties document. The 
    // default is 1.
    //
    private int _min = 1;
    
    //
    // The name of the property
    //
    private QName _qname = null;
    
    //
    // The base type of the property
    //
    private QName _typeName = XsdUtils.ANY_TYPE_QNAME;
    
    /**
     * 
     * @param qname
     *        The name of the new property.
     * 
     * @param definition
     *        The property's XML schema element (from the WS-RP document).
     *
     */
    public PropertySchemaDefinition(QName qname, Element definition)
    {
        if (qname == null)
            throw new NullPointerException(_MESSAGES.get("NullQName"));
        
        if (definition == null)
            throw new NullPointerException(_MESSAGES.get("NullPropertyElement"));
        
        _qname = qname;
        
        //
        // property minimum - optional
        //
        String attr = definition.getAttribute(XsdUtils.MIN_OCCURS);
        
        if (attr != null && attr.length() > 0)
            _min = Integer.parseInt(attr);
        
        //
        // property maximum - optional
        //
        attr = definition.getAttribute(XsdUtils.MAX_OCCURS);
        
        if (attr != null && attr.length() > 0)
        {
            boolean isUnbounded = attr.equals(XsdUtils.UNBOUNDED);
            
            if (isUnbounded)
                _max = ResourcePropertiesSchema.UNBOUNDED;
            
            else
                _max = Integer.parseInt(attr);
        }
                
        //
        // finally, scan the whole wsdl:types for the type definition
        //
        Document doc = definition.getOwnerDocument();
        Element typeDeclaration = WsdlUtils.getElementDeclaration(doc, _qname);
        
        if (typeDeclaration == null)
        {
            Object[] filler = { qname };
            throw new RuntimeException(_MESSAGES.get("NoTypeDeclaration", filler));
        }

        //
        // get the concrete/parent type of the property (if any)
        //        
        attr = typeDeclaration.getAttribute(XsdUtils.TYPE);
        
        if (attr != null && attr.length() > 0)
            _typeName = XmlUtils.parseQName(attr, typeDeclaration);
        
        //
        // property's nillable setting - optional
        //
        attr = typeDeclaration.getAttribute(XsdUtils.NILLABLE);
        
        if (attr != null && attr.length() > 0)
            _isNillable = Boolean.valueOf(attr).booleanValue(); 
    }
    
    public WsResourceCapability getCapability()
    {
        return _capability;
    }

    public int getMaxOccurs()
    {
        return _max;
    }

    public int getMinOccurs()
    {
        return _min;
    }
    
    public QName getPropertyName()
    {
        return _qname;
    }

    public QName getPropertyTypeName()
    {
        return _typeName;
    }
    
    public boolean isMaxUnbounded()
    {
        return getMaxOccurs() == ResourcePropertiesSchema.UNBOUNDED;
    }
    
    public boolean isNillable()
    {
        return _isNillable;
    }
    
    public void setCapability(WsResourceCapability capability)
    {
        //
        // make sure the user isn't trying to associate two capabilities with 
        // the same property
        //
        if (_capability != null)
        {
            Object[] filler = { getPropertyName(), _capability.getCapabilityURI(), capability.getCapabilityURI() };
            throw new RuntimeException(_MESSAGES.get("DuplicateCapability", filler));
        }
        
        _capability = capability;
    }
}
