/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.ws.resource.properties.set.impl;

import javax.xml.namespace.QName;

import org.w3c.dom.Element;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.resource.properties.impl.WsrpUtils;
import org.apache.muse.ws.resource.properties.set.SetRequestComponent;

/**
 *
 * AbstractSetRequestComponent is a the abstract base class for the 
 * WS-ResourceProperties SetResourceProperties commands. It provides the basic 
 * creation and execution mechanisms, as well as security.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public abstract class AbstractSetRequestComponent implements SetRequestComponent
{
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = 
        MessagesFactory.get(AbstractSetRequestComponent.class);

    private static final Element[] _EMPTY_VALUES = new Element[0];
    
    //
    // The name of the property to modify
    //
    private QName _qname = null;
    
    //
    // The token that the WS-RP container can use to determine if this 
    // request was created from within the resource instance or by 
    // an external request. The default is a unique, non-null token that 
    // will not map to any WS-RP container.
    //
    private Object _securityToken = new Object();
    
    //
    // The values (instances) of the property to use during modification
    //
    private Element[] _valuesXML = null;
    
    public int getNumberOfValues()
    {
        return _valuesXML.length;
    }
    
    public QName getPropertyName()
    {
        return _qname;
    }
    
    public Object getSecurityToken()
    {
        return _securityToken;
    }
    
    public Element getValue(int index)
    {
        return _valuesXML[index];
    }
    
    public Element[] getValues()
    {
        return _valuesXML;
    }
    
    public final void setPropertyName(QName qname)
    {
        if (qname == null)
            throw new NullPointerException(_MESSAGES.get("NullQName"));
        
        _qname = qname;
    }
    
    public final void setSecurityToken(Object securityToken)
    {
        _securityToken = securityToken;
    }
    
    public final void setValues(Object[] values)
    {
        if (values == null)
            throw new NullPointerException(_MESSAGES.get("NullValueArray"));
        
        if (values.length == 0)
        {
            _valuesXML = _EMPTY_VALUES;
            return;
        }
        
        //
        // we want to store the values as XML, and this is more complicated 
        // than it first appears...
        //
        
        QName qname = getPropertyName();
        
        //
        // first, we need to find out what type the objects are, and 
        // serialize them to XML accordingly. however, some references 
        // may be null, so we first have to find non-null reference to 
        // get its java type
        //
        
        int firstNonNull = -1;
        
        for (int n = 0; n < values.length && firstNonNull < 0; ++n)
            if (values[n] != null)
                firstNonNull = n;
        
        //
        // if all elements are null, just create a bunch of empty elements
        //
        if (firstNonNull < 0)
        {
            firstNonNull = 0;
            Element xml = XmlUtils.createElement(qname);
            
            for (int n = 0; n < values.length; ++n)
                values[n] = xml;                
        }
        
        //
        // otherwise, we can use the appropriate serializer to convert 
        // the objects into XML
        //
        
        Class type = values[firstNonNull].getClass();
        
        //
        // special case - for objects that are DOM Elements, just copy 
        // them over as-is (no re-creation)
        //
        if (Element.class.isAssignableFrom(type))
        {
            _valuesXML = new Element[values.length];
            
            for (int n = 0; n < values.length; ++n)
                _valuesXML[n] = XmlUtils.createElement(qname, values[n]);
        }
        
        else
        {
            try
            {
                _valuesXML = WsrpUtils.convertToElements(values, type, qname);
            }
            
            catch (Throwable error)
            {
                throw new RuntimeException(error.getMessage(), error);
            }
        }
    }
}
