/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.ws.resource.properties.set.impl;

import javax.xml.namespace.QName;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.resource.basefaults.BaseFault;
import org.apache.muse.ws.resource.properties.ResourcePropertyCollection;
import org.apache.muse.ws.resource.properties.WsrpConstants;

/**
 *
 * DeleteRequest is a serializer/deserializer for the WS-ResourceProperties 
 * Delete operation, which is part of SetResourceProperties.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class DeleteRequest extends AbstractSetRequestComponent
{
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = MessagesFactory.get(DeleteRequest.class);
    
    //
    // The name of this SetResourceProperties operation
    //
    public static final String OPERATION = "Delete";
    
    public DeleteRequest(Element request)
    {
        if (request == null)
            throw new NullPointerException(_MESSAGES.get("NullRequestElement"));
        
        String property = request.getAttribute(WsrpConstants.RESOURCE_PROPERTY);
        QName qname = XmlUtils.parseQName(property, request);
        
        setPropertyName(qname);
        setValues(new Object[0]);
    }
    
    public DeleteRequest(QName qname)
    {
        if (qname == null)
            throw new NullPointerException(_MESSAGES.get("NullQName"));
        
        setPropertyName(qname);
        setValues(new Object[0]);
    }
    
    public void execute(ResourcePropertyCollection properties)
        throws BaseFault
    {
        if (properties == null)
            throw new NullPointerException(_MESSAGES.get("NullProperties"));
        
        properties.deleteResourceProperty(getPropertyName(), getSecurityToken());
    }
    
    public String toString()
    {
        return XmlUtils.toString(toXML(), false);
    }
    
    public Element toXML()
    {
        return toXML(XmlUtils.createDocument());
    }
    
    public Element toXML(Document doc)
    {
        if (doc == null)
            throw new NullPointerException(_MESSAGES.get("NullDocument"));
        
        QName qname = getPropertyName();
        String prefix = qname.getPrefix();
        String uri = qname.getNamespaceURI();
        
        Element root = XmlUtils.createElement(doc, WsrpConstants.DELETE_QNAME);
        
        //
        // if the name of the property has a prefix/NS, we need to set the 
        // NS attribute in the request element
        //
        if (prefix != null && prefix.length() > 0)
            XmlUtils.setNamespaceAttribute(root, prefix, uri);
        
        String propertyName = XmlUtils.toString(qname);
        root.setAttribute(WsrpConstants.RESOURCE_PROPERTY, propertyName);
        
        return root;
    }
}
