/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.ws.resource.properties.set.impl;

import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.ws.resource.properties.WsrpConstants;
import org.apache.muse.ws.resource.properties.set.SetRequest;
import org.apache.muse.ws.resource.properties.set.SetRequestComponent;
import org.apache.muse.util.xml.XmlUtils;

/**
 *
 * SimpleSetRequest is a simple container that represents the WS-ResourceProperties 
 * SetResourceProperties operation's request content. It may contain one or 
 * more executable {@linkplain AbstractSetRequestComponent SetRequestComponents}.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class SimpleSetRequest implements SetRequest
{
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = MessagesFactory.get(SimpleSetRequest.class);
    
    //
    // The individual operations that insert, update, delete, etc.
    //
    private List _operations = new LinkedList();
    
    public void addRequestComponent(SetRequestComponent operation)
    {
        if (operation == null)
            throw new NullPointerException(_MESSAGES.get("NullSetRequestComponent"));
        
        _operations.add(operation);
    }
    
    public List getRequestComponents()
    {
        return Collections.unmodifiableList(_operations);
    }

    public String toString()
    {
        return XmlUtils.toString(toXML(), false);
    }
    
    public Element toXML()
    {
        return toXML(XmlUtils.createDocument());
    }
    
    public Element toXML(Document doc)
    {
        if (doc == null)
            throw new NullPointerException(_MESSAGES.get("NullDocument"));
        
        Element root = XmlUtils.createElement(doc, WsrpConstants.SET_QNAME);
        
        //
        // the SetResourceProperties XML is an aggregate of the XML for 
        // the concrete set operations (insert/delete/update/etc)
        //
        List ops = getRequestComponents();
        
        //
        // can't have empty set requests
        //
        if (ops.isEmpty())
            throw new IllegalStateException(_MESSAGES.get("EmptySetRequestBuilt"));
        
        Iterator i = ops.iterator();
        
        while (i.hasNext())
        {
            AbstractSetRequestComponent next = (AbstractSetRequestComponent)i.next();
            Element nextXML = next.toXML(doc);
            root.appendChild(nextXML);
        }
        
        return root;
    }
}
